use strict;
use warnings;
use Carp;
use Cwd qw(abs_path);
use File::Basename qw(dirname);
use POSIX::Run::Capture qw(:std);
use POSIX ":sys_wait_h";

my $dir = dirname(dirname(abs_path(__FILE__)));

our $catbin = $dir . '/runcap/t/genout';
our $input = $dir . '/runcap/t/INPUT';

sub mismatch {
    my ($what, $exp, $got) = @_;
    diag("$what mismatch: $exp <=> $got");
}

sub test_stream {
    my ($cap, $kw, $fd) = @_;
    
    if (exists($kw->{length}) && $kw->{length} != $cap->length($fd)) {
	mismatch("stdout length", $kw->{length}, $cap->length($fd));
	return 0;
    }
    if (exists($kw->{nlines}) && $kw->{nlines} != $cap->nlines($fd)) {
	mismatch("number of stdout lines", $kw->{nlines}, $cap->nlines($fd));
	return 0;
    }
    if (exists($kw->{content})) {
	my $lines = $cap->get_lines($fd);
	my $content = join('',@$lines);
	if ($kw->{content} ne $content) {
	    diag("stdout content mismatch");
	    return 0;
	}
    }
    return 1;
}
    
sub TestCapture {
    my ($argv, %kw) = @_;

    $kw{result} = 0 unless exists $kw{result};
    $kw{code}   = 0 unless exists $kw{code};

    my $cap = new POSIX::Run::Capture @{$argv};

    return 0 unless $cap;

    my $res = $cap->run;

    unless ($kw{result} == $res) {
	mismatch("result code", $res, $kw{result});
	return 0;
    }

    unless (WIFEXITED($cap->status)) {
	diag("unexpected termination status: ".$cap->status);
	return 0;
    }

    unless (WEXITSTATUS($cap->status) == $kw{code}) {
	mismatch("exit code", WEXITSTATUS($cap->status), $kw{code});
	return 0;
    }

    if (exists($kw{stdout}) && !test_stream($cap, $kw{stdout}, SD_STDOUT)) {
	return 0;
    }

    if (exists($kw{stderr}) && !test_stream($cap, $kw{stderr}, SD_STDERR)) {
	return 0;
    }
    
    return 1;
}

our $content =<<'_INLINE_';
CHAPTER I. Down the Rabbit-Hole

Alice was beginning to get very tired of sitting by her sister on the
bank, and of having nothing to do: once or twice she had peeped into the
book her sister was reading, but it had no pictures or conversations
in it, 'and what is the use of a book,' thought Alice 'without
pictures or conversations?'

So she was considering in her own mind (as well as she could, for the
hot day made her feel very sleepy and stupid), whether the pleasure of
making a daisy-chain would be worth the trouble of getting up and picking
the daisies, when suddenly a White Rabbit with pink eyes ran close by her.

There was nothing so very remarkable in that; nor did Alice think it
so very much out of the way to hear the Rabbit say to itself, 'Oh
dear! Oh dear! I shall be late!' (when she thought it over afterwards,
it occurred to her that she ought to have wondered at this, but at the
time it all seemed quite natural); but when the Rabbit actually took a
watch out of its waistcoat-pocket, and looked at it, and then hurried on,
Alice started to her feet, for it flashed across her mind that she had
never before seen a rabbit with either a waistcoat-pocket, or a watch
to take out of it, and burning with curiosity, she ran across the field
after it, and fortunately was just in time to see it pop down a large
rabbit-hole under the hedge.

In another moment down went Alice after it, never once considering how
in the world she was to get out again.

The rabbit-hole went straight on like a tunnel for some way, and then
dipped suddenly down, so suddenly that Alice had not a moment to think
about stopping herself before she found herself falling down a very
deep well.

Either the well was very deep, or she fell very slowly, for she had plenty
of time as she went down to look about her and to wonder what was going
to happen next. First, she tried to look down and make out what she was
coming to, but it was too dark to see anything; then she looked at the
sides of the well, and noticed that they were filled with cupboards
and book-shelves; here and there she saw maps and pictures hung upon
pegs. She took down a jar from one of the shelves as she passed; it was
labelled 'ORANGE MARMALADE', but to her great disappointment it was
empty: she did not like to drop the jar for fear of killing somebody,
so managed to put it into one of the cupboards as she fell past it.

'Well!' thought Alice to herself, 'after such a fall as this,
I shall think nothing of tumbling down stairs! How brave they'll all
think me at home! Why, I wouldn't say anything about it, even if I
fell off the top of the house!' (Which was very likely true.)

Down, down, down. Would the fall never come to an end! 'I wonder how
many miles I've fallen by this time?' she said aloud. 'I must be
getting somewhere near the centre of the earth. Let me see: that would be
four thousand miles down, I think--' (for, you see, Alice had learnt
several things of this sort in her lessons in the schoolroom, and though
this was not a very good opportunity for showing off her knowledge,
as there was no one to listen to her, still it was good practice to
say it over) '--yes, that's about the right distance--but then I
wonder what Latitude or Longitude I've got to?' (Alice had no idea
what Latitude was, or Longitude either, but thought they were nice grand
words to say.)

Presently she began again. 'I wonder if I shall fall right through
the earth! How funny it'll seem to come out among the people that
walk with their heads downward! The Antipathies, I think--' (she was
rather glad there was no one listening, this time, as it didn't sound
at all the right word) '--but I shall have to ask them what the name
of the country is, you know. Please, Ma'am, is this New Zealand or
Australia?' (and she tried to curtsey as she spoke--fancy curtseying
as you're falling through the air! Do you think you could manage
it?) 'And what an ignorant little girl she'll think me for asking! No,
it'll never do to ask: perhaps I shall see it written up somewhere.'
_INLINE_
;
